@@ -11,19 +11,24 @@
 import random
 import string
 import hashlib
-import urllib
 
+import pytz
 from wtforms import Form, TextField, IntegerField, SelectField, validators, \
     PasswordField
 from wtfrecaptcha.fields import RecaptchaField
 from werkzeug import redirect, abort
 
-from nereid import request, url_for, render_template, login_required, flash
+from nereid import request, url_for, render_template, login_required, flash, \
+    jsonify
+from nereid.contrib import gravatar
 from nereid.globals import session, current_app
+from nereid.signals import registration
+from nereid.templating import render_email
 from trytond.model import ModelView, ModelSQL, fields
 from trytond.pyson import Eval, Bool, Not
 from trytond.transaction import Transaction
 from trytond.config import CONFIG
+from trytond.tools import get_smtp_server
 
 from .i18n import _, get_translations
 
@@ -46,8 +51,8 @@ class RegistrationForm(Form):
 
     if 're_captcha_public' in CONFIG.options:
         captcha = RecaptchaField(
-            public_key=CONFIG.options['re_captcha_public'], 
-            private_key=CONFIG.options['re_captcha_private'], 
+            public_key=CONFIG.options['re_captcha_public'],
+            private_key=CONFIG.options['re_captcha_private'],
             secure=True
         )
 
@@ -113,26 +118,26 @@ class AdditionalDetails(ModelSQL, ModelView):
     _name = "address.additional_details"
     _description = __doc__
     _rec_name = 'value'
-    
+
     def get_types(self):
         """
-        Wrapper to convert _get_types dictionary 
+        Wrapper to convert _get_types dictionary
         into a `list of tuple` for the use of Type Selection field
-        
+
         This hook will scan all methods which start with _type_address_extend
-        
+
         Your hook extension should look like:
-                
+
         def _type_address_extend_<name>(self, cursor, user, context=None):
             return {
                         '<name>': '<value>'
             }
-        
+
         An example from ups:
-        
+
         return {'type': 'value'
             }
-        
+
         :return: the list of tuple for Selection field
         """
         type_dict = {}
@@ -153,13 +158,13 @@ class AdditionalDetails(ModelSQL, ModelView):
 
     def default_active(self):
         return True
-        
+
     def _type_address_extend_default(self):
         return {
             'dob': 'Date of Birth',
             'other': 'Other',
         }
-    
+
 AdditionalDetails()
 
 
@@ -176,7 +181,7 @@ class Address(ModelSQL, ModelView):
     #: This field is retained only for legacy purposes.
     #: Additional details is now directly stored on the user object
     additional_details = fields.One2Many(
-        'address.additional_details', 
+        'address.additional_details',
         'address', 'Additional Details', states=STATES
     )
     email = fields.Char('Email')
@@ -252,10 +257,19 @@ class Address(ModelSQL, ModelView):
 Address()
 
 
+class Party(ModelSQL, ModelView):
+    "Party"
+    _name = 'party.party'
+
+    nereid_users = fields.One2Many('nereid.user', 'party', 'Nereid Users')
+
+Party()
+
+
 class NereidUser(ModelSQL, ModelView):
     """
     Nereid Users
-    
+
     The Users were address records in versions before 0.3
 
     .. versionadded:: 0.3
@@ -267,6 +281,28 @@ class NereidUser(ModelSQL, ModelView):
     party = fields.Many2One('party.party', 'Party', required=True,
             ondelete='CASCADE', select=1)
 
+    #: It is recommended to use this field to display the name of the user, as
+    #: the behaviour of how the name of the user will be displayed depends on 
+    #: the context (b2b or b2c)
+    #:
+    #: In a b2c context, the nereid_user is the same as party, so the name
+    #: field from nereid_user (inherited from party) will do the job of being
+    #: both the name of the user and of the party (in the context of Tryton)
+    #:
+    #: In a b2b context the party becomes the entity with which Tryton does
+    #: business with and that party could have multiple users. Though the
+    #: data model allows this, the name of the user will be confusing without
+    #: a display name as the default name of the user in the case will be that
+    #: of the company.
+    #:
+    #: The default implementation of display_name uses it for a b2c context
+    #: and is a proxy for the name of the party
+    display_name = fields.Function(
+        fields.Char('Display Name'),
+        'get_display_name', setter='set_display_name',
+        searcher='search_display_name'
+    )
+
     #: The email of the user is also the login name/username of the user
     email = fields.Char("e-Mail", select=1)
 
@@ -291,6 +327,40 @@ class NereidUser(ModelSQL, ModelView):
     #     Company is mandatory
     company = fields.Many2One('company.company', 'Company', required=True)
 
+    timezone = fields.Selection(
+        [(x, x) for x in pytz.common_timezones], 'Timezone', translate=False
+    )
+
+    permissions = fields.Many2Many('nereid.permission-nereid.user',
+        'nereid_user', 'permission', 'Permissions')
+
+    def get_permissions(self, user):
+        """
+        Returns all the permissions as a list of names
+
+        :param user: Browse Record of the user
+        """
+        # TODO: Cache this value for each user to avoid hitting the database
+        # everytime.
+        return frozenset([p.value for p in user.permissions])
+
+    def has_permissions(self, nereid_user, permissions):
+        """Check if the user has required permissions for access
+
+        :param permissions: A set/frozenset of permission values/keywords
+
+        :return: True/False
+        """
+        if not isinstance(permissions, (set, frozenset)):
+            permissions = frozenset(permissions)
+        current_user_permissions = self.get_permissions(nereid_user)
+        if permissions.issubset(current_user_permissions):
+            return True
+        return False
+
+    def default_timezone(self):
+        return "UTC"
+
     def default_company(self):
         return Transaction().context.get('company') or False
 
@@ -318,6 +388,37 @@ class NereidUser(ModelSQL, ModelView):
                     'Invalid Activation Code'
         return self.write(user.id, {'activation_code': None})
 
+    def get_display_name(self, ids, name):
+        """
+        Returns the name of the party as the name of the user
+
+        :param ids: List of ids
+        :param name: Name of the field
+        """
+        res = {}
+        for user in self.browse(ids):
+            res[user.id] = user.party.name
+        return res
+
+    def set_display_name(self, ids, name, value):
+        """
+        Do nothing for now as the name field and the display name are the same
+
+        :param ids: List of ids
+        :param name: Name of the field
+        :param value: The value of the field
+        """
+        return True
+
+    def search_display_name(self, name, clause):
+        """
+        Alter the display_name search pattern to search in the name field
+
+        :param name: Name of the field for which the search is being done
+        :param clause: The search clause from the domain expression
+        """
+        return [('name') + clause[1:]]
+
     def get_registration_form(self):
         """
         Returns a registration form for use in the site
@@ -357,11 +458,15 @@ class NereidUser(ModelSQL, ModelView):
             else:
                 user_id = self.create({
                     'name': registration_form.name.data,
+                    'display_name': registration_form.name.data,
                     'email': registration_form.email.data,
                     'password': registration_form.password.data,
                     'company': request.nereid_website.company.id,
                     })
                 self.create_act_code(user_id)
+                registration.send(user_id)
+                user = self.browse(user_id)
+                self.send_activation_email(user)
                 flash(
                     _('Registration Complete. Check your email for activation')
                 )
@@ -371,6 +476,24 @@ class NereidUser(ModelSQL, ModelView):
 
         return render_template('registration.jinja', form=registration_form)
 
+    def send_activation_email(self, nereid_user):
+        """
+        Send an activation email to the user
+
+        :param nereid_user: The browse record of the user
+        """
+        email_message = render_email(
+            CONFIG['smtp_from'], nereid_user.email, _('Account Activation'),
+            text_template = 'emails/activation-text.jinja',
+            html_template = 'emails/activation-html.jinja',
+            nereid_user = nereid_user
+        )
+        server = get_smtp_server()
+        server.sendmail(
+            CONFIG['smtp_from'], [nereid_user.email], email_message.as_string()
+        )
+        server.quit()
+
     @login_required
     def change_password(self):
         """
@@ -394,7 +517,7 @@ class NereidUser(ModelSQL, ModelView):
 
             if password_sha == user.password:
                 self.write(
-                    request.nereid_user.id, 
+                    request.nereid_user.id,
                     {'password': form.password.data}
                 )
                 flash(
@@ -405,7 +528,7 @@ class NereidUser(ModelSQL, ModelView):
                 return redirect(url_for('nereid.website.login'))
             else:
                 flash(_("The current password you entered is invalid"))
-        
+
         return render_template(
             'change-password.jinja', change_password_form=form
         )
@@ -413,13 +536,13 @@ class NereidUser(ModelSQL, ModelView):
     @login_required
     def new_password(self):
         """Create a new password
-        
+
         .. tip::
 
-            Unlike change password this does not demand the old password. 
-            And hence this method will check in the session for a parameter 
-            called allow_new_password which has to be True. This acts as a 
-            security against attempts to POST to this method and changing 
+            Unlike change password this does not demand the old password.
+            And hence this method will check in the session for a parameter
+            called allow_new_password which has to be True. This acts as a
+            security against attempts to POST to this method and changing
             password.
 
             The allow_new_password flag is popped on successful saving
@@ -434,7 +557,7 @@ class NereidUser(ModelSQL, ModelView):
                 abort(403)
 
             self.write(
-                request.nereid_user.id, 
+                request.nereid_user.id,
                 {'password': form.password.data}
             )
             session.pop('allow_new_password')
@@ -473,10 +596,10 @@ class NereidUser(ModelSQL, ModelView):
 
     def create_act_code(self, user_id, code_type="new"):
         """Create activation code
-            
-        A 12 character activation code indicates reset while 16 
+
+        A 12 character activation code indicates reset while 16
         character activation code indicates a new registration
-        
+
         :param user_id: ID of the User
         :param code_type:   "new" for new activation code
                             "reset" for resetting existing account
@@ -491,7 +614,7 @@ class NereidUser(ModelSQL, ModelView):
 
     def reset_account(self):
         """
-        Reset the password for the user. 
+        Reset the password for the user.
 
         .. tip::
             This does NOT reset the password, but just creates an activation
@@ -509,12 +632,32 @@ class NereidUser(ModelSQL, ModelView):
                 return render_template('reset-password.jinja')
 
             self.create_act_code(user_ids[0], "reset")
+            user = self.browse(user_ids[0])
+            self.send_reset_email(user)
             flash(_('An email has been sent to your account for resetting'
                 ' your credentials'))
             return redirect(url_for('nereid.website.login'))
 
         return render_template('reset-password.jinja')
 
+    def send_reset_email(self, nereid_user):
+        """
+        Send an account reset email to the user
+
+        :param nereid_user: The browse record of the user
+        """
+        email_message = render_email(
+            CONFIG['smtp_from'], nereid_user.email, _('Account Password Reset'),
+            text_template = 'emails/reset-text.jinja',
+            html_template = 'emails/reset-html.jinja',
+            nereid_user = nereid_user
+        )
+        server = get_smtp_server()
+        server.sendmail(
+            CONFIG['smtp_from'], [nereid_user.email], email_message.as_string()
+        )
+        server.quit()
+
     def authenticate(self, email, password):
         """Assert credentials and if correct return the
         browse record of the user
@@ -609,33 +752,135 @@ class NereidUser(ModelSQL, ModelView):
                         For example a unisex avatar
         :param size: The size for the image
         """
-        if kwargs.get('https', request.scheme == 'https'):
-            url = 'https://secure.gravatar.com/avatar/%s?'
-        else:
-            url = 'http://www.gravatar.com/avatar/%s?'
-        url = url % hashlib.md5(user.email.lower()).hexdigest()
+        return gravatar.url(user.email, **kwargs)
 
-        params = []
-        default = kwargs.get('default', None)
-        if default:
-            params.append(('d', default))
+    def aslocaltime(self, naive_date, user=None):
+        """
+        Returns a localized time using `pytz.astimezone` method.
 
-        size = kwargs.get('size', None)
-        if size:
-            params.append(('s', str(size)))
+        :param naive_date: a naive datetime (datetime with no timezone
+                           information), which is assumed to be the UTC time.
+        :param user: If a user is provided, the timezone of the user is taken
+                     to build the offset or the user from nereid context is
+                     used. The value if provided must be a browse record.
+        :return: A datetime object with local time
+        """
+        if user is None:
+            user = request.nereid_user
 
-        return url + urllib.urlencode(params)
+        utc_date = pytz.utc.localize(naive_date)
+
+        if not user.timezone:
+            return utc_date
+
+        user_tz = pytz.timezone(user.timezone)
+        if user_tz == pytz.utc:
+            return utc_date
+
+        return utc_date.astimezone(user_tz)
 
 NereidUser()
 
 
-class EmailTemplate(ModelSQL, ModelView):
-    'add `url_for` to the template context'
-    _name = 'electronic_mail.template'
+class ContactMechanismForm(Form):
+    type = SelectField('Type', [validators.Required()])
+    value = TextField('Value', [validators.Required()])
+    comment = TextField('Comment')
 
-    def template_context(self, record):
-        context = super(EmailTemplate, self).template_context(record)
-        context['url_for'] = url_for
-        return context
 
-EmailTemplate()
+class ContactMechanism(ModelSQL, ModelView):
+    """
+    Allow modification of contact mechanisms
+    """
+    _name = "party.contact_mechanism"
+
+    def get_form(self):
+        """
+        Returns the contact mechanism form
+        """
+        from trytond.modules.party import contact_mechanism
+        form = ContactMechanismForm(request.form)
+        form.type.choices = contact_mechanism._TYPES
+        return form
+
+    @login_required
+    def add(self):
+        """
+        Adds a contact mechanism to the party's contact mechanisms
+        """
+        form = self.get_form()
+        if form.validate():
+            self.create({
+                'party': request.nereid_user.party.id,
+                'type': form.type.data,
+                'value': form.value.data,
+                'comment': form.comment.data,
+            })
+            if request.is_xhr:
+                return jsonify({'success': True})
+            return redirect(request.referrer)
+
+        if request.is_xhr:
+            return jsonify({'success': False})
+        else:
+            for field, messages in form.errors:
+                flash("<br>".join(messages), "Field %s" % field)
+            return redirect(request.referrer)
+
+    @login_required
+    def remove(self):
+        """
+        :param record_id: Delete the contat mechanism with the given ID
+        """
+        record_id = request.form.get('record_id', type=int)
+        if not record_id:
+            abort(404)
+
+        record = self.browse(record_id)
+        if not record:
+            abort(404)
+        if record.party == request.nereid_user.party:
+            self.delete(record_id)
+        else:
+            abort(403)
+        if request.is_xhr:
+            return jsonify({
+                'success': True
+            })
+        return redirect(request.referrer)
+
+ContactMechanism()
+
+
+class Permission(ModelSQL, ModelView):
+    "Nereid Permissions"
+    _name = 'nereid.permission'
+    _description = __doc__
+
+    name = fields.Char('Name', required=True, select=True)
+    value  = fields.Char('Value', required=True, select=True)
+    nereid_users = fields.Many2Many('nereid.permission-nereid.user',
+        'permission', 'nereid_user', 'Nereid Users'
+    )
+
+    def __init__(self):
+        super(Permission, self).__init__()
+        self._sql_constraints += [
+            ('unique_value', 'UNIQUE(value)',
+                'Permissions must be unique by value'),
+            ]
+
+Permission()
+
+
+class UserPermission(ModelSQL):
+    "Nereid User Permissions"
+    _name = 'nereid.permission-nereid.user'
+    _description = __doc__
+
+    permission = fields.Many2One('nereid.permission', 'Permission',
+        ondelete='CASCADE', select=True, required=True)
+    nereid_user = fields.Many2One('nereid.user', 'User',
+        ondelete='CASCADE', select=True, required=True)
+
+UserPermission()
